<?php

namespace Shopping\Customized\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Shopping\Customized\Http\Requests\CustomizedRequest;
use Shopping\Customized\Interfaces\CustomizedRepositoryInterface;
use Shopping\Customized\Models\Customized;

/**
 * Resource controller class for customized.
 */
class CustomizedResourceController extends BaseController
{

    /**
     * Initialize customized resource controller.
     *
     * @param type CustomizedRepositoryInterface $customized
     *
     * @return null
     */
    public function __construct(CustomizedRepositoryInterface $customized)
    {
        parent::__construct();
        $this->repository = $customized;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Shopping\Customized\Repositories\Criteria\CustomizedResourceCriteria::class);
    }

    /**
     * Display a list of customized.
     *
     * @return Response
     */
    public function index(CustomizedRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Shopping\Customized\Repositories\Presenter\CustomizedPresenter::class)
                ->$function();
        }

        $customizeds = $this->repository->paginate();

        return $this->response->title(trans('customized::customized.names'))
            ->view('customized::customized.index', true)
            ->data(compact('customizeds'))
            ->output();
    }

    /**
     * Display customized.
     *
     * @param Request $request
     * @param Model   $customized
     *
     * @return Response
     */
    public function show(CustomizedRequest $request, Customized $customized)
    {

        if ($customized->exists) {
            $view = 'customized::customized.show';
        } else {
            $view = 'customized::customized.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('customized::customized.name'))
            ->data(compact('customized'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new customized.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CustomizedRequest $request)
    {

        $customized = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('customized::customized.name')) 
            ->view('customized::customized.create', true) 
            ->data(compact('customized'))
            ->output();
    }

    /**
     * Create new customized.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CustomizedRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $customized                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('customized::customized.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('customized/customized/' . $customized->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/customized/customized'))
                ->redirect();
        }

    }

    /**
     * Show customized for editing.
     *
     * @param Request $request
     * @param Model   $customized
     *
     * @return Response
     */
    public function edit(CustomizedRequest $request, Customized $customized)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('customized::customized.name'))
            ->view('customized::customized.edit', true)
            ->data(compact('customized'))
            ->output();
    }

    /**
     * Update the customized.
     *
     * @param Request $request
     * @param Model   $customized
     *
     * @return Response
     */
    public function update(CustomizedRequest $request, Customized $customized)
    {
        try {
            $attributes = $request->all();

            $customized->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('customized::customized.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('customized/customized/' . $customized->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('customized/customized/' . $customized->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the customized.
     *
     * @param Model   $customized
     *
     * @return Response
     */
    public function destroy(CustomizedRequest $request, Customized $customized)
    {
        try {

            $customized->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('customized::customized.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('customized/customized/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('customized/customized/' . $customized->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple customized.
     *
     * @param Model   $customized
     *
     * @return Response
     */
    public function delete(CustomizedRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('customized::customized.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('customized/customized'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/customized/customized'))
                ->redirect();
        }

    }

    /**
     * Restore deleted customizeds.
     *
     * @param Model   $customized
     *
     * @return Response
     */
    public function restore(CustomizedRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('customized::customized.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/customized/customized'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/customized/customized/'))
                ->redirect();
        }

    }

}
